from io import StringIO
from loguru import logger
import psycopg2
from AbstractSinkHandler import AbstractSinkHandler
from PgDbHandle import PgDbHandle

class TsDbSinkException(Exception):
    def __init__(self, code, msg):
        super().__init__()
        self.code = code
        self.msg = msg

class AbstractTsDbSinkHandler(AbstractSinkHandler):
    def __init__(self, config, name, inQueue, experiment):
        super().__init__(config, name, inQueue, experiment)
        logger.info("constructor called")
        self.dbh = PgDbHandle(config)

    def init(self):
        logger.info("init")
        self.dbh.connectDb()

    def deinit(self):
        logger.info("deinit")
        self.dbh.disconnectDb()

    def copy(self, table, columns, data):
        try:
            stringIO = StringIO(data)
            with self.dbh.connection as conn:
                with conn.cursor() as cursor:
                    cursor.copy_from(stringIO, table, columns=columns)
        except psycopg2.Error as e:
            raise TsDbSinkException(e.pgcode, str(e))
    
    def insert(self, table, columns, data):
        placeholders = ['%s'] * len(columns)
        stmt = f"insert into {table} ({','.join(columns)}) values ({','.join(placeholders)})"
        logger.debug(f"{stmt=}")
        try:
            with self.dbh.connection as conn:
                with conn.cursor() as cursor:
                    cursor.execute(stmt, data)
        except psycopg2.Error as e:
            raise TsDbSinkException(e.pgcode, str(e))
