from loguru import logger
from AbstractTsDbSinkHandler import TsDbSinkException, AbstractTsDbSinkHandler
import iso8601


class FlowRigCmdTsDbSinkHandler(AbstractTsDbSinkHandler):
    def __init__(self, config, name, inQueue):
        super().__init__(config, name, inQueue, 'dummyexperiment')
        logger.info("constructor called")

    def sinkAction(self, dataObject):
        logger.info("database write operation start")
        try:
            # payload format: check dataformat.txt
            # "ts": "yyyy-MM-dd'T'HH-mm-ss.SSSZ", 
            # "en": "Name of Experiment",
            # "ec": "Comment discribing the experiment",
            # "es": "start/stop"
            ts = iso8601.parse_date(dataObject.payload['ts'])
            en = dataObject.payload['en']
            ec = dataObject.payload['ec']
            es = dataObject.payload['es']

            self.insert('flowrigcmd', ('time', 'experiment', 'description', 'command'), (ts, en, ec, es))
            logger.info("database write operation done")
        except iso8601.ParseError as e:
            logger.error(f"unable to parse timestamp, items dropped: {e}")
        except KeyError as e:
            logger.error(f"missing attribute in payload, items dropped: {e}")
        except TsDbSinkException as e:
            logger.error(f"error {e.code} when talking to database, items dropped: {e.msg}")


